/** 
 * @name PHPIDS
 * @uses http://php-ids.org/
 *
 * @author Stefanie Janine Stoelting, mail@stefanie-stoelting.de
 * @modification Thomas Jakobi, thomas.jakobi@partout.info
 * @changes Use MODx logging/Use MODx email settings/use MODx Plugin configuration (don't need to patch config.ini and/or plugin code)
 * @package PHPIDS
 * @license LGPL
 * @since 2009/11/13
 * @version 0.6.3.1 *
 * <strong>0.6.2</strong> A plugin to include PHPIDS into MODx to log and prevent intrusions
 * First download PHPDS from its homepage and load it up to your webspace.
 * If you install PHPIDS into a public path, you should remove webaccess to the
 * lib path, for example with the attached .htaccess file.
 *
 * This plugin is based on an article published by Heise Verlag:
 * Getting started with the PHPIDS intrusion detection system
 * http://www.h-online.com/security/Getting-started-with-the-PHPIDS-intrusion-detection-system--/features/113163
 *
 * Requirements: MODx 0.9.6 or newer, PHP 5.1.2 or newer, PHPIDS 0.6.3.1
 *
 * On page Systemevents check the entry OnWebPageInit
 *
 * For a test just call your site with the following parameters: index.php?id=1&test=">XXX
 *
 *
 * 
 * Plugin Configuration
 *
 * &useMODxBasePath=Use MODx base path?;list;true,false;true &basePath=PHPIDS path?;string;assets/lib/phpids/lib &useLogFile=Use log file?;list;true,false;false &useEmail=Use email?;list;true,false;true &useDatabase=Use database?;list;true,false;true &useMODxTablePrefix=Use MODx table prefix?;list;true,false;true &useMODxLog=Use MODx Log?;list;true,false;true &logFromImpact=Start logging from impact?;int;5 &sendMailFromImpact=Start sending mails from impact, high (around 15) or very high (around 25-50) impact?;int;15 &idsRecipients=Mail resipients (separated by comma);text; &idsSubject=Mail subject <br />(keep empty to use default);text; &SMTPServer=SMPT server;text; &SMTPPort=SMPT port <br />(default is 25, leave empty to use default);text; &SMTPAccount=SMPT account;text; &SMTPPassword=SMPT password;text; &mailFrom=E-Mail From Name<br> Only used with SMTP;text;PHPIDS  
 *
 **/

$useMODxBasePath    = (isset($useMODxBasePath) && (strtolower($useMODxBasePath) != 'true') && ($useMODxBasePath != '1' )) ? false : true;
$useLogFile         = (isset($useLogFile) && (strtolower($useLogFile) != 'true') && ($useLogFile != '1' )) ? false : true;
$useEmail           = (isset($useEmail) && (strtolower($useEmail) != 'true') && ($useEmail != '1' )) ? false : true;
$mailFrom           = (isset($mailFrom) && (!empty($mailFrom))) ? $mailFrom : $modx->config['emailsender'];
$useDatabase        = (isset($useDatabase) && (strtolower($useDatabase) != 'true') && ($useDatabase != '1' )) ? false : true;
$useMODxLog         = (isset($useMODxLog) && (strtolower($useMODxLog) != 'true') && ($useMODxLog != '1' )) ? false : true;
$useMODxTablePrefix = (isset($useMODxTablePrefix) && (strtolower($useMODxTablePrefix) != 'true') && ($useMODxTablePrefix != '1' )) ? false : true;
$logFromImpact      = isset($logFromImpact) ? $logFromImpact : 5;
$sendMailFromImpact = isset($sendMailFromImpact) ? $sendMailFromImpact : 25;
$idsRecipients      = isset($idsRecipients) ? $idsRecipients: $modx->config['emailsender'];
$idsSubject         = isset($idsSubject) ? $idsSubject: $modx->config['site_name'] . ': PHPIDS Plugin detected an intrusion attempt!';
$SMTPServer         = isset($SMTPServer) ? $SMTPServer : '';
$SMTPPort           = isset($SMTPPort) ? $SMTPPort : '25';
$SMTPAccount        = isset($SMTPAccount) ? $SMTPAccount : '';
$SMTPPassword       = isset($SMTPPassword) ? $SMTPPassword : '';

$useSMTP            = !empty($SMTPServer) && !empty($SMTPPort) && !empty($SMTPAccount) && !empty($SMTPPassword);

if ($useMODXBasePath) {
  $basePath           = isset($basePath)? $modx->config['base_path']. 'assets/lib/phpids/lib': '';
} else {
  $basePath           = isset($basePath)? $basePath: '';
}

if (empty($basePath)) {

  $modx->logEvent(0, 2, 'PHPIDS Plugin: PHPIDS path not set!');
  // Exeption is only shown, on backend login
  if (isset($_SESSION['mgrValidated'])? $_SESSION['mgrValidated']: false == 1) {
    $modx->messageQuit ('PHPIDS Plugin: PHPIDS path not set!');
  }
  
} else {

  set_include_path(
     get_include_path()
     . PATH_SEPARATOR
     . $basePath
    );


  require_once 'IDS/Init.php';
  try {
    $request = array(
      'REQUEST' => $_REQUEST,
      'GET' => $_GET,
      'POST' => $_POST,
      'COOKIE' => $_COOKIE
    );

    $init = IDS_Init::init($basePath . '/IDS/Config/Config.ini');

    $init->config['General']['base_path'] = $basePath . '/IDS/';
    $init->config['General']['use_base_path'] = true;
    
    // E-Mail configuration
    $init->config['Logging']['recipients'] = $idsRecipients;
    $init->config['Logging']['subject'] = $idsSubject;
    $init->config['Logging']['header'] = 'From: ' . $modx->config['emailsender'];
    $init->config['Caching']['caching'] = 'none';

    // Database configuration
    $init->config['Logging']['wrapper'] = 'mysql:host='.$modx->db->config['host'].';port=3306;dbname='.trim($modx->db->config['dbase'], '`');
    $init->config['Logging']['user'] = $modx->db->config['user'];
    $init->config['Logging']['password'] = $modx->db->config['pass'];
    
    if ($useMODxTablePrefix) {
      $init->config['Logging']['table'] = $modx->db->config['table_prefix'] . 'phpids_intrusions';
    } else {
      $init->config['Logging']['table'] = 'phpids_intrusions';    
    }
    

    $ids = new IDS_Monitor($request, $init);
    $result = $ids->run();
    $impact = $result->getImpact();

    if (!$result->isEmpty()) {

      require_once 'IDS/Log/Composite.php';

      $compositeLog = new IDS_Log_Composite();

      if ($useLogFile && $impact >= $logFromImpact) {

        require_once 'IDS/Log/File.php';
        $compositeLog->addLogger(IDS_Log_File::getInstance($init));

      }


      if ($useEmail && $impact >= $sendMailFromImpact) {

        if ($useSMTP) {

          require_once($modx->config['base_path']. 'manager/includes/controls/class.phpmailer.php');
          $cPHPMailer = new PHPMailer();
          
          try {
            // Configure SMTP
            $cPHPMailer->IsSMTP();
            $cPHPMailer->SMTPAuth = true; //Turn on SMTP authentication
            $cPHPMailer->Host = $SMTPServer;
            $cPHPMailer->Port = $SMTPPort;
            $cPHPMailer->Username = $SMTPAccount;
            $cPHPMailer->Password = $SMTPPassword;
            $cPHPMailer->CharSet = 'UTF-8';
            
            // Add recipients
            $recipients = explode(',', $idsRecipients);
            foreach ($recipients as $recipient) {
              $cPHPMailer->AddAddress($recipient);
            }
            $cPHPMailer->FromName = $mailFrom;
            $cPHPMailer->From =  $modx->config['emailsender'];
            
            $cPHPMailer->Subject = $idsSubject;
            
            $format = "The following attack has been detected by PHPIDS\n\n"
                     ."IP: %s \n\n"
                     ."Date: %s \n\n";

            $mailBody = sprintf($format,
                          $_SERVER['REMOTE_ADDR'],
                          date('c')
                        );
            
            $mailBody .= str_replace('<br/>', "\n", $result);
            
            $cPHPMailer->Body = $mailBody;
            
            $cPHPMailer->Send();
            
          } catch (Exception $e) {
            
            $modx->logEvent(0, 3, sprintf( 'An error occured while creating SMTP mail: %s', $e->getMessage() ), 'PHPIDS Plugin');
            
          }

          

        } else {
          
          require_once 'IDS/Log/Email.php';
          $compositeLog->addLogger(IDS_Log_Email::getInstance($init));

        }

      }

      if ($useDatabase && $impact >= $logFromImpact) {
        
        if ($useMODxTablePrefix) {
          $tableName =  $modx->db->config['table_prefix'] . 'phpids_intrusions';
        } else {
          $tableName = 'phpids_intrusions';
        }
        
        $sql = 'CREATE TABLE IF NOT EXISTS `' . $tableName . '` (
                  `id` int(11) unsigned NOT null auto_increment,
                  `name` varchar(128) NOT null,
                  `value` text NOT null,
                  `page` varchar(255) NOT null,
                  `tags` varchar(128) NOT null,
                  `ip` varchar(15) NOT null,
                  `impact` int(11) unsigned NOT null,
                  `origin` varchar(15) NOT null,
                  `created` datetime NOT null,
                  PRIMARY KEY  (`id`)
                ) ENGINE=MyISAM ;';
        $modx->db->query($sql);

        require_once 'IDS/Log/Database.php';
        $compositeLog->addLogger(IDS_Log_Database::getInstance($init));

      }

      if ($useMODxLog && $impact >= $logFromImpact) {

        $modx->logEvent( 0, 2, sprintf( 'IP: %s<br /><br />', $_SERVER['REMOTE_ADDR'] ) . $result, sprintf('PHPIDS Intrusion Alert! Impact: %s', $impact), 'PHPIDS Plugin' );
          
      }

      $compositeLog->execute($result);
    }

  } catch (Exception $e) {

    $modx->logEvent(0, 2, sprintf( 'An error occured: %s', $e->getMessage() ), 'PHPIDS Plugin' );
    // Exeption is only shown, on backend login
    if (isset($_SESSION['mgrValidated'])? $_SESSION['mgrValidated']: false == 1) {
      $modx->messageQuit ( sprintf ( 'An error occured: %s', $e->getMessage() ) );
    }

  }
}
